@@ -6,10 +6,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import error, util
-
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, csinfo, cmdcore, cmdui, status, resolve
 from tortoisehg.hgqt import qscilib, thgrepo, messageentry, commit, wctxcleaner
 
@@ -120,7 +118,7 @@ class BasePage(QWizardPage):
         self.wizard().setOption(QWizard.NoDefaultButton, False)
 
     def canExit(self):
-        if len(self.repo.parents()) == 2:
+        if len(self.repo[None].parents()) == 2:
             main = _('Do you want to exit?')
             text = _('To finish merging, you must commit '
                      'the working directory.\n\n'
@@ -221,8 +219,8 @@ class SummaryPage(BasePage):
         wdbox.addWidget(force_chk)
 
         ### discard option
-        discard_chk = QCheckBox(_('Discard all changes from merge target '
-                                  '(other) revision'))
+        discard_chk = QCheckBox(_('Discard all changes from the other '
+                                  'revision'))
         self.registerField('discard', discard_chk)
         self.layout().addWidget(discard_chk)
 
@@ -294,7 +292,7 @@ class SummaryPage(BasePage):
         self.completeChanged.emit()
         self.refreshFinished.emit()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'skip':
             self.wizard().next()
@@ -329,26 +327,32 @@ class MergePage(BasePage):
 
     def currentPage(self):
         super(MergePage, self).currentPage()
-        if self.field('discard').toBool():
-            # '.' is safer than self.localrev, in case the user has
-            # pulled a fast one on us and updated from the CLI
-            cmdline = ['debugsetparents', '.', hglib.tounicode(self._otherrev)]
-        else:
-            cmdline = ['merge', '--verbose']
-            if self.field('force').toBool():
-                cmdline.append('--force')
-            tool = self.field('autoresolve').toBool() and 'merge' or 'fail'
-            cmdline += ['--tool=internal:' + tool]
-            cmdline.append(hglib.tounicode(self._otherrev))
-
-        if len(self.repo.parents()) == 1:
-            self._cmdlog.clearLog()
-            self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
-            sess.commandFinished.connect(self.onCommandFinished)
-            sess.outputReceived.connect(self._cmdlog.appendLog)
-        else:
+        if len(self.repo[None].parents()) > 1:
             self.mergecomplete = True
             self.completeChanged.emit()
+            return
+
+        discard = self.field('discard').toBool()
+        rev = hglib.tounicode(self._otherrev)
+        cfgs = []
+        if discard:
+            tool = ':local'
+            # disable changed/deleted prompt because we'll revert changes
+            cfgs.append('ui.interactive=False')
+        else:
+            tool = self.field('autoresolve').toBool() and ':merge' or ':fail'
+        cmdlines = [hglib.buildcmdargs('merge', rev, verbose=True, tool=tool,
+                                       force=self.field('force').toBool(),
+                                       config=cfgs)]
+        if discard:
+            # revert files added/removed at other side
+            cmdlines.append(hglib.buildcmdargs('revert', rev='.', all=True))
+
+        self._cmdlog.clearLog()
+        self._cmdsession = sess = self._repoagent.runCommandSequence(cmdlines,
+                                                                     self)
+        sess.commandFinished.connect(self.onCommandFinished)
+        sess.outputReceived.connect(self._cmdlog.appendLog)
 
     def isComplete(self):
         'should Next button be sensitive?'
@@ -394,7 +398,7 @@ class MergePage(BasePage):
                 self.tryAutoAdvance(True)
             self.completeChanged.emit()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
             dlg = resolve.ResolveDialog(self._repoagent, self)
@@ -538,7 +542,7 @@ class CommitPage(BasePage):
         self.msgEntry.setText(commit.mergecommitmessage(self.repo))
         self.msgEntry.moveCursorToEnd()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, cmd):
         if cmd == 'view':
             dlg = status.StatusDialog(self._repoagent, [], {}, self)
@@ -546,13 +550,14 @@ class CommitPage(BasePage):
             self.refresh()
 
     def isComplete(self):
-        return len(self.repo.parents()) == 2 and len(self.msgEntry.text()) > 0
+        return (len(self.repo[None].parents()) == 2 and
+                len(self.msgEntry.text()) > 0)
 
     def validatePage(self):
         if not self._cmdsession.isFinished():
             return False
 
-        if len(self.repo.parents()) == 1:
+        if len(self.repo[None].parents()) == 1:
             # commit succeeded, repositoryChanged() called wizard().next()
             if self.field('skiplast').toBool():
                 self.wizard().close()
@@ -566,31 +571,13 @@ class CommitPage(BasePage):
         self.setTitle(_('Committing...'))
         self.setSubTitle(_('Please wait while committing merged files.'))
 
-        message = unicode(self.msgEntry.text())
-        cmdline = ['commit', '--verbose', '--message', message, '--user', user]
-        if self.opts.get('recurseinsubrepos'):
-            cmdline.append('--subrepos')
-        try:
-            date = self.opts.get('date')
-            if date:
-                util.parsedate(date)
-                dcmd = ['--date', hglib.tounicode(date)]
-            else:
-                dcmd = []
-        except error.Abort, e:
-            if e.hint:
-                err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
-                                            hglib.tounicode(e.hint))
-            else:
-                err = hglib.tounicode(str(e))
-            qtlib.WarningMsgBox(_('TortoiseHg Merge Commit'),
-                _('Error creating interpreting commit date (%s).\n'
-                  'Using current date instead.'), err)
-            dcmd = []
-
-        cmdline += dcmd
-
-        commandlines = [cmdline]
+        opts = {'verbose': True,
+                'message': self.msgEntry.text(),
+                'user': user,
+                'subrepos': bool(self.opts.get('recurseinsubrepos')),
+                'date': hglib.tounicode(self.opts.get('date')),
+                }
+        commandlines = [hglib.buildcmdargs('commit', **opts)]
         pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
         if pushafter:
             cmd = ['push', hglib.tounicode(pushafter)]
@@ -604,7 +591,7 @@ class CommitPage(BasePage):
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
-        if len(self.repo.parents()) == 1:
+        if len(self.repo[None].parents()) == 1:
             if not self._cmdsession.isFinished():
                 # call self.wizard().next() after the current command finishes
                 self.delayednext = True
