@@ -10,10 +10,10 @@ import errno
 
 from mercurial import extensions, error, util
 
-from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, cmdcore, cmdui, update, revdetails
 from tortoisehg.hgqt.qtlib import geticon
 from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -38,8 +38,7 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
         self.show_internal_branches = False
 
         repoagent.configChanged.connect(self.configChanged)
-        repoagent.repositoryChanged.connect(self.repositoryChanged)
-        repoagent.workingBranchChanged.connect(self.workingBranchChanged)
+        repoagent.repositoryChanged.connect(self.refresh)
 
         # Build child widgets
 
@@ -52,6 +51,7 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
 
         def Toolbar():
             tb = QToolBar(_("Patch Branch Toolbar"), self)
+            tb.setIconSize(qtlib.toolBarIconSize())
             tb.setEnabled(True)
             tb.setObjectName("toolBar_patchbranch")
             tb.setFloatable(False)
@@ -75,7 +75,7 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
             #self.actionReapply.triggered.connect(self.reapply_clicked)
 
             self.actionPNew = a = QWidgetAction(self)
-            a.setIcon(geticon("fileadd")) #STOCK_NEW
+            a.setIcon(geticon("hg-add")) #STOCK_NEW
             a.setToolTip(_('Start a new patch branch'))
             tb.addAction(self.actionPNew)
             self.actionPNew.triggered.connect(self.pnew_clicked)
@@ -136,6 +136,7 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
         self.repo.thginvalidate()
         self.refresh()
 
+    @pyqtSlot()
     def refresh(self):
         """
         Refresh the list of patches.
@@ -353,9 +354,8 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
         """
         if self.pbranch is None:
             return False
-        self.repo.incrementBusyCount()
         self.pbranch.cmdnew(self.repo.ui, self.repo, patch_name)
-        self.repo.decrementBusyCount()
+        self._repoagent.pollStatus()
         return True
 
     def pmerge(self, patch_name=None):
@@ -436,7 +436,7 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
     def update_sensitivity(self):
         """ Update the sensitivity of entire UI """
         in_pbranch = True #TODO
-        is_merge = len(self.repo.parents()) > 1
+        is_merge = len(self.repo[None].parents()) > 1
         self.actionPMerge.setEnabled(in_pbranch)
         self.actionBackport.setEnabled(in_pbranch)
         self.actionReapply.setEnabled(True)
@@ -507,14 +507,6 @@ class PatchBranchWidget(QWidget, qtlib.TaskWidget):
     def configChanged(self):
         pass
 
-    @pyqtSlot()
-    def repositoryChanged(self):
-        self.refresh()
-
-    @pyqtSlot()
-    def workingBranchChanged(self):
-        self.refresh()
-
     def pmerge_clicked(self):
         self.pmerge()
 
@@ -827,7 +819,7 @@ class PNewDialog(QDialog):
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
         self.setWindowFlags(Qt.Window)
-        self.setWindowIcon(qtlib.geticon("fileadd"))
+        self.setWindowIcon(qtlib.geticon("hg-add"))
         self.setWindowTitle(_('New Patch Branch'))
 
         def AddField(var, label, optional=False):
