@@ -21,7 +21,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from mercurial import extensions, scmutil
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import cmdcore, qtlib
 from tortoisehg.hgqt.postreview_ui import Ui_PostReviewDialog
 from tortoisehg.hgqt.hgemail import _ChangesetsModel
@@ -257,7 +257,8 @@ class PostReviewDialog(QDialog):
         if (len(self.selectedRevs) > 1):
             #Set the parent to the revision below the last one on the list
             #so all checked revisions are included in the request
-            opts['parent'] = str(self.selectedRevs[0] - 1)
+            ctx = self.repo[self.selectedRevs[0]]
+            opts['parent'] = str(ctx.p1().rev())
 
         # Always use the upstream repo to determine the parent diff base
         # without the diff uploaded to review board dies
@@ -335,8 +336,6 @@ class PostReviewDialog(QDialog):
 
             return args
 
-        hglib.loadextension(self.ui, 'reviewboard')
-
         opts = self.postReviewOpts()
 
         revstr = str(self.selectedRevs.pop())
@@ -388,7 +387,7 @@ class PostReviewDialog(QDialog):
         self.writeSettings()
         super(PostReviewDialog, self).accept()
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def _captureOutput(self, msg, label):
         if label != 'control':
             self._cmdoutputs.append(unicode(msg))
