@@ -85,10 +85,10 @@ def openlocalurl(path):
     '''
 
     if isinstance(path, str):
-        path = QString(hglib.tounicode(path))
-    elif isinstance(path, unicode):
-        path = QString(path)
-    if os.name == 'nt' and path.startsWith('\\\\'):
+        path = hglib.tounicode(path)
+    else:
+        path = unicode(path)
+    if os.name == 'nt' and path.startswith('\\\\'):
         # network share, special handling because of qt bug 13359
         # see http://bugreports.qt.nokia.com/browse/QTBUG-13359
         qurl = QUrl()
@@ -255,7 +255,7 @@ def openshell(root, reponame, ui=None):
                    _('A terminal shell must be configured'))
 
 
-def isdarktheme(palette=None):
+def isDarkTheme(palette=None):
     """True if white-on-black color scheme is preferable"""
     if not palette:
         palette = QApplication.palette()
@@ -486,7 +486,7 @@ def descriptionhtmlizer(ui):
 
 _iconcache = {}
 
-if getattr(sys, 'frozen', False):
+if getattr(sys, 'frozen', False) and os.name == 'nt':
     def iconpath(f, *insidef):
         return posixpath.join(':/icons', f, *insidef)
 else:
@@ -596,9 +596,26 @@ def getcheckboxpixmap(state, bgcolor, widget):
     style.drawPrimitive(style.PE_IndicatorCheckBox, option, painter)
     return pix
 
+
+# On machines with a retina display running OSX (i.e. "darwin"), most icons are
+# too big because Qt4 does not support retina displays very well.
+# To fix that we let users force tortoishg to use smaller icons by setting a
+# THG_RETINA environment variable to True (or any value that mercurial parses
+# as True.
+IS_RETINA = False
+if sys.platform == 'darwin':
+    IS_RETINA = util.parsebool(os.environ.get('THG_RETINA', '0'))
+
+
+def _fixIconSizeForRetinaDisplay(s):
+    if IS_RETINA and s > 1:
+        s /= 2
+    return s
+
 def smallIconSize():
     style = QApplication.style()
     s = style.pixelMetric(QStyle.PM_SmallIconSize)
+    s = _fixIconSizeForRetinaDisplay(s)
     return QSize(s, s)
 
 def toolBarIconSize():
@@ -609,8 +626,18 @@ def toolBarIconSize():
     else:
         style = QApplication.style()
     s = style.pixelMetric(QStyle.PM_ToolBarIconSize)
+    s = _fixIconSizeForRetinaDisplay(s)
     return QSize(s, s)
 
+def listviewRetinaIconSize():
+    return QSize(16, 16)
+
+def treeviewRetinaIconSize():
+    return QSize(16, 16)
+
+def barRetinaIconSize():
+    return QSize(10, 10)
+
 class ThgFont(QObject):
     changed = pyqtSignal(QFont)
     def __init__(self, name):
@@ -1229,6 +1256,8 @@ def _configuredusername(ui):
         and not os.environ.get('EMAIL')):
         return None
     try:
+        if ui.configbool('ui', 'askusername'):
+            ui.setconfig('ui', 'askusername', False)
         return ui.username()
     except error.Abort:
         return None
@@ -1323,7 +1352,7 @@ class PaletteSwitcher(object):
     def __init__(self, targetwidget):
         self._targetwref = weakref.ref(targetwidget)  # avoid circular ref
         self._defaultpalette = targetwidget.palette()
-        if not isdarktheme(self._defaultpalette):
+        if not isDarkTheme(self._defaultpalette):
             filterbgcolor = QColor('#FFFFB7')
         else:
             filterbgcolor = QColor('darkgrey')
