@@ -10,7 +10,7 @@ import os, sys
 from mercurial import util
 
 from tortoisehg.util import hglib, shlib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, status, cmdcore, cmdui, lfprompt
 
 from PyQt4.QtCore import *
@@ -21,7 +21,7 @@ LABELS = { 'add': (_('Checkmark files to add'), _('Add')),
            'revert': (_('Checkmark files to revert'), _('Revert')),
            'remove': (_('Checkmark files to remove'), _('Remove')),}
 
-ICONS = { 'add': 'fileadd',
+ICONS = { 'add': 'hg-add',
            'forget': 'hg-remove',
            'revert': 'hg-revert',
            'remove': 'hg-remove',}
@@ -185,10 +185,16 @@ class QuickOpDialog(QDialog):
                                 _('No operation to perform'),
                                 parent=self)
             return
-        self.repo.lfstatus = True
-        repostate = self.repo.status()
-        self.repo.lfstatus = False
         if self.command == 'remove':
+            self.repo.lfstatus = True
+            try:
+                repostate = self.repo.status()
+            except (EnvironmentError, util.Abort), e:
+                qtlib.WarningMsgBox(_('Unable to read repository status'),
+                                    hglib.tounicode(str(e)), parent=self)
+                return
+            finally:
+                self.repo.lfstatus = False
             if not self.chk.isChecked():
                 modified = repostate[0]
                 selmodified = []
