@@ -149,9 +149,10 @@ class RebaseDialog(QDialog):
     def _onCheckFinished(self, clean):
         if not clean:
             self.rebasebtn.setEnabled(False)
-            txt = _('Before rebase, you must <a href="commit">'
-                    '<b>commit</b></a> or <a href="discard">'
-                    '<b>discard</b></a> changes.')
+            txt = _('Before rebase, you must '
+                    '<a href="commit"><b>commit</b></a>, '
+                    '<a href="shelve"><b>shelve</b></a> to patch, '
+                    'or <a href="discard"><b>discard</b></a> changes.')
         else:
             self.rebasebtn.setEnabled(True)
             txt = _('You may continue the rebase')
