@@ -24,7 +24,7 @@ Also, enable the plugin with the following in mercurial.ini::
 from mercurial import error
 
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import cmdui, csinfo, qtlib
 
 from PyQt4.QtCore import *
@@ -163,9 +163,9 @@ class RemoteUpdateWidget(cmdui.AbstractCmdWidget):
     ### Signal Handlers ###
 
     def show_options(self, visible):
-        self.push_chk.setShown(visible)
-        self.newbranch_chk.setShown(visible)
-        self.force_chk.setShown(visible)
+        self.push_chk.setVisible(visible)
+        self.newbranch_chk.setVisible(visible)
+        self.force_chk.setVisible(visible)
 
 
 def createRemoteUpdateDialog(repoagent, rev=None, parent=None):
