@@ -554,7 +554,7 @@ class _fi(object):
             return self.visible()
 
 INFO = (
-({'name': 'general', 'label': 'TortoiseHg', 'icon': 'thg_logo'}, (
+({'name': 'general', 'label': 'TortoiseHg', 'icon': 'thg'}, (
     _fi(_('UI Language'), 'tortoisehg.ui.language',
         (genDeferredCombo, i18n.availablelanguages),
         _('Specify your preferred user interface language (restart needed)'),
@@ -579,6 +579,11 @@ INFO = (
           'section of your Mercurial configuration files.  If left '
           'unspecified, TortoiseHg will use the first applicable tool '
           'it finds.')),
+    _fi(_('CLI Editor'), 'ui.editor',
+        (genEditableDeferredCombo, findEditors),
+        _('The editor used by Mercurial command line commands to '
+          'collect multiline input from the user.  Most notably, '
+          'commit messages.')),
     _fi(_('Shell'), 'tortoisehg.shell',
         (genEditableDeferredCombo, findTerminals),
         _('Specify the command to launch your preferred terminal shell '
@@ -612,7 +617,8 @@ INFO = (
           'A value of zero implies no limit.  Default: 1024 (1MB)')),
     _fi(_('Fork GUI'), 'tortoisehg.guifork', genBoolRBGroup,
         _('When running from the command line, fork a background '
-          'process to run graphical dialogs.  Default: True'),
+          'process to run graphical dialogs. This setting is ignored when '
+          'TortoiseHg runs as an OS X application bundle.  Default: True'),
         globalonly=True),
     _fi(_('Full Path Title'), 'tortoisehg.fullpath', genBoolRBGroup,
         _('Show a full directory path of the repository in the dialog title '
@@ -631,7 +637,7 @@ INFO = (
         globalonly=True),
     )),
 
-({'name': 'log', 'label': _('Workbench'), 'icon': 'menulog'}, (
+({'name': 'log', 'label': _('Workbench'), 'icon': 'hg-log'}, (
     _fi(_('Single Workbench Window'), 'tortoisehg.workbench.single',
         genBoolRBGroup,
         _('Select whether you want to have a single workbench window. '
@@ -721,7 +727,7 @@ INFO = (
           'This option is expected to be removed if the performance issue is '
           'solved.')),
     )),
-({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'menucommit'},
+({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'hg-commit'},
  (
     _fi(_('Username'), 'ui.username', genEditCombo,
         _('Name associated with commits.  The common format is:<br>'
@@ -833,7 +839,7 @@ INFO = (
           'Default: "ssh" or "TortoisePlink.exe -ssh -2" (Windows)')),
     )),
 
-({'name': 'web', 'label': _('Server'), 'icon': 'proxy'}, (
+({'name': 'web', 'label': _('Server'), 'icon': 'hg-serve'}, (
     _fi(_('<b>Repository Details:</b>'), None, genSpacer, ''),
     _fi(_('Name'), 'web.name', genEditCombo,
         _('Repository name to use in the web interface, and by TortoiseHg '
@@ -1097,6 +1103,13 @@ INFO = (
           'stored')),
     )),
 
+({'name': 'simplelock', 'label': _('Simplelock'), 'icon': 'thg-password',
+  'extension': 'simplelock'}, (
+    _fi(_('Lock Clone'), 'simplelock.repo', genEditCombo,
+        _('Location of local clone of organizational lock repository.<p>'
+          'This repository must contain a "locked" text file')),
+    )),
+
 ({'name': 'largefiles', 'label': _('Largefiles'), 'icon': 'kiln',
   'extension': 'largefiles'}, (
     _fi(_('Patterns'), 'largefiles.patterns', genEditCombo,
@@ -1171,7 +1184,7 @@ class SettingsDialog(QDialog):
         self.setWindowTitle(_('TortoiseHg Settings'))
         self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint
                             | Qt.WindowMaximizeButtonHint)
-        self.setWindowIcon(qtlib.geticon('settings_repo'))
+        self.setWindowIcon(qtlib.geticon('thg-repoconfig'))
 
         if not hasattr(wconfig.config(), 'write'):
             qtlib.ErrorMsgBox(_('Iniparse package not found'),
@@ -1202,8 +1215,10 @@ class SettingsDialog(QDialog):
         self._activeformidx = configrepo and CONF_REPO or CONF_GLOBAL
         self.conftabs = QTabWidget()
         layout.addWidget(self.conftabs)
+        if qtlib.IS_RETINA:
+            self.conftabs.setIconSize(qtlib.barRetinaIconSize())
         utab = SettingsForm(rcpath=scmutil.userrcpath(), focus=focus)
-        self.conftabs.addTab(utab, qtlib.geticon('settings_user'),
+        self.conftabs.addTab(utab, qtlib.geticon('thg-userconfig'),
                              _("%s's global settings") % username())
         utab.restartRequested.connect(self._pushRestartRequest)
 
@@ -1235,7 +1250,7 @@ class SettingsDialog(QDialog):
 
             reporcpath = os.sep.join([repo.root, '.hg', 'hgrc'])
             rtab = SettingsForm(rcpath=reporcpath, focus=focus)
-            self.conftabs.addTab(rtab, qtlib.geticon('settings_repo'),
+            self.conftabs.addTab(rtab, qtlib.geticon('thg-repoconfig'),
                                  _('%s repository settings')
                                  % repoagent.shortName())
             rtab.restartRequested.connect(self._pushRestartRequest)
@@ -1252,10 +1267,10 @@ class SettingsDialog(QDialog):
         self.conftabs.currentChanged.connect(self._currentFormChanged)
 
     def isDirty(self):
-        return util.any(self.conftabs.widget(i).isDirty()
-                        for i in xrange(self.conftabs.count()))
+        return any(self.conftabs.widget(i).isDirty()
+                   for i in xrange(self.conftabs.count()))
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _pushRestartRequest(self, key):
         self._restartreqs.add(unicode(key))
 
@@ -1268,7 +1283,7 @@ class SettingsDialog(QDialog):
                                'for the following changes to take effect:'),
                              ', '.join(sorted(self._restartreqs)))
             self._restartreqs.clear()
-        return util.all(results)
+        return all(results)
 
     def canExit(self):
         if self.isDirty():
@@ -1317,7 +1332,7 @@ class SettingsDialog(QDialog):
 class SettingsForm(QWidget):
     """Widget for each settings file"""
 
-    restartRequested = pyqtSignal(unicode)
+    restartRequested = pyqtSignal(str)
 
     def __init__(self, rcpath, focus=None, parent=None, readonly=False):
         super(SettingsForm, self).__init__(parent)
@@ -1340,8 +1355,10 @@ class SettingsForm(QWidget):
         self.fnedit = QLineEdit()
         self.fnedit.setReadOnly(True)
         self.fnedit.setFrame(False)
-        self.fnedit.setFocusPolicy(Qt.NoFocus)
-        self.fnedit.setStyleSheet('QLineEdit { background: transparent; }')
+        self.fnedit.setFocusPolicy(Qt.ClickFocus)
+        p = self.fnedit.palette()
+        p.setColor(QPalette.Base, Qt.transparent)
+        self.fnedit.setPalette(p)
         edit = QPushButton(_('Edit File'))
         edit.clicked.connect(self.editClicked)
         self.editbtn = edit
@@ -1357,6 +1374,8 @@ class SettingsForm(QWidget):
         layout.addLayout(bothbox, 8)
         pageList = QListWidget()
         pageList.setResizeMode(QListView.Fixed)
+        if qtlib.IS_RETINA:
+            pageList.setIconSize(qtlib.listviewRetinaIconSize())
         stack = QStackedWidget()
         bothbox.addWidget(pageList, 0)
         bothbox.addWidget(stack, 1)
