@@ -7,7 +7,7 @@
 
 import os
 
-from mercurial import hg, util, error, context, merge, scmutil
+from mercurial import hg, util, error, context, scmutil
 
 from tortoisehg.util import hglib
 from tortoisehg.util.i18n import _
@@ -39,15 +39,15 @@ class StatusWidget(QWidget):
     '''Working copy status widget
        SIGNALS:
        progress()                   - for progress bar
-       showMessage(unicode)         - for status bar
-       titleTextChanged(QString)    - for window title
+       showMessage(str)             - for status bar
+       titleTextChanged(str)        - for window title
     '''
-    progress = pyqtSignal(QString, object, QString, QString, object)
-    titleTextChanged = pyqtSignal(QString)
-    linkActivated = pyqtSignal(QString)
-    showMessage = pyqtSignal(unicode)
-    fileDisplayed = pyqtSignal(QString, QString)
-    grepRequested = pyqtSignal(unicode, dict)
+    progress = pyqtSignal(str, object, str, str, object)
+    titleTextChanged = pyqtSignal(str)
+    linkActivated = pyqtSignal(str)
+    showMessage = pyqtSignal(str)
+    fileDisplayed = pyqtSignal(str, str)
+    grepRequested = pyqtSignal(str, dict)
     runCustomCommandRequested = pyqtSignal(str, list)
 
     def __init__(self, repoagent, pats, opts, parent=None, checkable=True,
@@ -393,7 +393,7 @@ class StatusWidget(QWidget):
             self.updateModel(self.refthread.wctx, self.refthread.wstatus,
                              self.refthread.patchecked)
         self.refthread = None
-        if len(self.repo.parents()) > 1:
+        if len(self.repo[None].parents()) > 1:
             # nuke partial selections if wctx has a merge in-progress
             self.partials = {}
         match = self.le.text()
@@ -401,7 +401,7 @@ class StatusWidget(QWidget):
             self.setFilter(match)
 
     # better to handle error in reloadComplete in place of separate signal?
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def reloadFailed(self, msg):
         qtlib.ErrorMsgBox(_('Failed to refresh'), msg, parent=self)
 
@@ -421,7 +421,7 @@ class StatusWidget(QWidget):
                 qtlib.WarningMsgBox(_('No appropriate files'),
                                     _('No files found for this operation'),
                                     parent=self)
-        ms = merge.mergestate(self.repo)
+        ms = hglib.readmergestate(self.repo)
         tm = WctxModel(self._repoagent, wctx, wstatus, ms, self.pctx,
                        self.savechecks, self.opts, checked, self,
                        checkable=self.checkable, defcheck=self.defcheck)
@@ -494,7 +494,7 @@ class StatusWidget(QWidget):
             self.opts[val.name] = s in status
         self.refreshWctx()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def setFilter(self, match):
         model = self.tv.model()
         if model:
@@ -523,7 +523,7 @@ class StatusWidget(QWidget):
             self.checkAllNoneBtn.setTristate(state == Qt.PartiallyChecked)
             self.checkAllNoneBtn.setCheckState(state)
 
-    @pyqtSlot(QString, bool)
+    @pyqtSlot(str, bool)
     def checkToggled(self, wfile, checked):
         'user has toggled a checkbox, update partial chunk selection status'
         wfile = hglib.fromunicode(wfile)
@@ -619,7 +619,7 @@ class StatusWidget(QWidget):
 class StatusThread(QThread):
     '''Background thread for generating a workingctx'''
 
-    showMessage = pyqtSignal(QString)
+    showMessage = pyqtSignal(str)
 
     def __init__(self, repo, pctx, pats, opts, parent=None):
         super(StatusThread, self).__init__()
@@ -699,7 +699,7 @@ class WctxFileTree(QTreeView):
 
 class WctxModel(QAbstractTableModel):
     checkCountChanged = pyqtSignal()
-    checkToggled = pyqtSignal(QString, bool)
+    checkToggled = pyqtSignal(str, bool)
 
     def __init__(self, repoagent, wctx, wstatus, ms, pctx, savechecks, opts,
                  checked, parent, checkable=True, defcheck='MAR!S'):
@@ -1016,22 +1016,22 @@ class StatusType(object):
         self.trname = trname
 
 statusTypes = {
-    'M' : StatusType('modified', 'menucommit.ico', _('%s is modified'),
+    'M' : StatusType('modified', 'hg-modified', _('%s is modified'),
                      'status.modified', _('modified')),
-    'A' : StatusType('added', 'fileadd.ico', _('%s is added'),
+    'A' : StatusType('added', 'hg-add', _('%s is added'),
                      'status.added', _('added')),
-    'R' : StatusType('removed', 'filedelete.ico', _('%s is removed'),
+    'R' : StatusType('removed', 'hg-removed', _('%s is removed'),
                      'status.removed', _('removed')),
-    '?' : StatusType('unknown', 'shelve.ico', _('%s is not tracked (unknown)'),
+    '?' : StatusType('unknown', '', _('%s is not tracked (unknown)'),
                      'status.unknown', _('unknown')),
-    '!' : StatusType('deleted', 'menudelete.ico',
+    '!' : StatusType('deleted', '',
                      _('%s is deleted by non-hg command, but still tracked'),
                      'status.deleted', _('missing')),
-    'I' : StatusType('ignored', 'ignore.ico', _('%s is ignored'),
+    'I' : StatusType('ignored', '', _('%s is ignored'),
                      'status.ignored', _('ignored')),
     'C' : StatusType('clean', '', _('%s is not modified (clean)'),
                      'status.clean', _('clean')),
-    'S' : StatusType('subrepo', 'thg-subrepo.ico', _('%s is a dirty subrepo'),
+    'S' : StatusType('subrepo', 'thg-subrepo', _('%s is a dirty subrepo'),
                      'status.subrepo', _('subrepo')),
 }
 
@@ -1077,7 +1077,7 @@ class StatusFilterActionGroup(QObject):
     @pyqtSlot(str)
     def setStatus(self, text):
         """Set the status text"""
-        assert util.all(c in self._TYPES for c in text)
+        assert all(c in self._TYPES for c in text)
         for c in self._TYPES:
             self._actions[c].setChecked(c in text)
 
