@@ -39,12 +39,12 @@ _extractnodeids = re.compile(r'^\s*([0-9a-f]{40})$', re.MULTILINE).findall
 class SyncWidget(QWidget, qtlib.TaskWidget):
     newCommand = pyqtSignal(cmdcore.CmdSession)
     outgoingNodes = pyqtSignal(object)
-    incomingBundle = pyqtSignal(QString, QString)
-    showMessage = pyqtSignal(unicode)
+    incomingBundle = pyqtSignal(str, str)
+    showMessage = pyqtSignal(str)
     pullCompleted = pyqtSignal()
     pushCompleted = pyqtSignal()
 
-    switchToRequest = pyqtSignal(QString)
+    switchToRequest = pyqtSignal(str)
 
     def __init__(self, repoagent, parent=None):
         QWidget.__init__(self, parent)
@@ -173,6 +173,12 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.urlentry.textChanged.connect(self.urlChanged)
         self.urlentry.returnPressed.connect(self.saveclicked)
         tbar.addWidget(self.urlentry)
+        tbar.addWidget(qtlib.Spacer(2, 2))
+
+        self.browsebutton = QPushButton(_('Browse...'))
+        self.browsebutton.setAutoDefault(False)
+        self.browsebutton.clicked.connect(self._browseUrl)
+        tbar.addWidget(self.browsebutton)
 
         # even though currentRowChanged fires pathSelected, clicked signal is
         # also connected to it. otherwise urlentry won't be updated when the
@@ -433,7 +439,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def canExit(self):
         return self._cmdsession.isFinished()
 
-    @pyqtSlot(QPoint, QString, QString, bool)
+    @pyqtSlot(QPoint, str, str, bool)
     def menuRequest(self, point, url, alias, editable):
         'menu event emitted by one of the two URL lists'
         if not self.cmenu:
@@ -528,6 +534,14 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             self.curalias = hglib.fromunicode(dlg.aliasentry.text())
             self.reload()
 
+    @pyqtSlot()
+    def _browseUrl(self):
+        FD = QFileDialog
+        caption = _("Select repository")
+        path = FD.getExistingDirectory(self, caption, self.urlentry.text())
+        if path:
+            self.urlentry.setText(QDir.toNativeSeparators(path))
+
     def secureclicked(self):
         if not parseurl(self.currentUrl()).host:
             qtlib.WarningMsgBox(_('No host specified'),
@@ -971,7 +985,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             # the bundle, and let the user accept or reject them
             self.inclicked()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def removeAlias(self, alias):
         alias = hglib.fromunicode(alias)
         fn = self.repo.join('hgrc')
@@ -1460,8 +1474,8 @@ are expanded in the filename.'''))
 
 
 class PathsTree(QTreeView):
-    removeAlias = pyqtSignal(QString)
-    menuRequest = pyqtSignal(QPoint, QString, QString, bool)
+    removeAlias = pyqtSignal(str)
+    menuRequest = pyqtSignal(QPoint, str, str, bool)
 
     def __init__(self, parent, editable):
         QTreeView.__init__(self, parent)
