@@ -8,14 +8,12 @@
 import os
 import subprocess
 import stat
-import shutil
 import threading
-import tempfile
 import re
 
-from mercurial import hg, util, error, match, scmutil, copies
+from mercurial import util, error, match, scmutil, copies
 
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib
 from tortoisehg.hgqt import qtlib
 
@@ -102,10 +100,13 @@ def snapshot(repo, files, ctx):
         try:
             if not os.path.isdir(destdir):
                 os.makedirs(destdir)
-            data = repo.wwritedata(wfn, ctx[wfn].data())
+            fctx = ctx[wfn]
+            data = repo.wwritedata(wfn, fctx.data())
             f = open(dest, 'wb')
             f.write(data)
             f.close()
+            if 'x' in fctx.flags():
+                util.setflags(dest, False, True)
             if ctx.rev() is None:
                 fns_and_mtime.append((dest, repo.wjoin(fn), 
                                     os.lstat(dest).st_mtime))
@@ -432,7 +433,7 @@ class FileSelectionDialog(QDialog):
             combo.setCurrentIndex(defrow)
 
             list.currentRowChanged.connect(self.updateToolSelection)
-            combo.currentIndexChanged['QString'].connect(self.onToolSelected)
+            combo.currentIndexChanged[str].connect(self.onToolSelected)
             self.toolCombo = combo
 
         BB = QDialogButtonBox
@@ -477,10 +478,10 @@ class FileSelectionDialog(QDialog):
         mod_a, add_a, rem_a = sa
         for f in sorted(mod_a | add_a | rem_a):
             status = get_status(f, mod_a, add_a, rem_a)
-            row = QString('%s %s' % (status, hglib.tounicode(f)))
+            row = '%s %s' % (status, hglib.tounicode(f))
             self.list.addItem(row)
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onToolSelected(self, tool):
         'user selected a tool from the tool combo'
         tool = hglib.fromunicode(tool)
