@@ -154,7 +154,7 @@ class Workbench(QMainWindow):
         newaction(_("&Close Repository"), self.closeCurrentRepoTab,
                   shortcut='Close', enabled='repoopen', menu='file')
         newseparator(menu='file')
-        newaction(_('&Settings'), self.editSettings, icon='settings_user',
+        newaction(_('&Settings'), self.editSettings, icon='thg-userconfig',
                   shortcut='Preferences', menu='file')
         newseparator(menu='file')
         newaction(_("E&xit"), self.close, shortcut='Quit', menu='file')
@@ -214,7 +214,7 @@ class Workbench(QMainWindow):
         # note that 'grep' and 'search' are equivalent
         taskdefs = {
             'commit': ('hg-commit', _('&Commit')),
-            'pbranch': ('branch', _('&Patch Branch')),
+            'pbranch': ('hg-branch', _('&Patch Branch')),
             'log': ('hg-log', _("Revision &Details")),
             'grep': ('hg-grep', _('&Search')),
             'sync': ('thg-sync', _('S&ynchronize')),
@@ -294,12 +294,17 @@ class Workbench(QMainWindow):
 
         newseparator(toolbar='edit')
         menuSync = self.menuRepository.addMenu(_('S&ynchronize'))
+        a = newaction(_("&Lock File..."), self._repofwd('lockTool'),
+                    icon='thg-password', enabled='repoopen',
+                    menu='repository', toolbar='edit',
+                    tooltip=_('Lock or unlock files'))
+        self.lockToolAction = a
         newseparator(menu='repository')
         newaction(_("&Update..."), self._repofwd('updateToRevision'),
                   icon='hg-update', enabled='repoopen',
                   menu='repository', toolbar='edit',
                   tooltip=_('Update working directory or switch revisions'))
-        newaction(_("&Shelve..."), self._repofwd('shelve'), icon='shelve',
+        newaction(_("&Shelve..."), self._repofwd('shelve'), icon='hg-shelve',
                   enabled='repoopen', menu='repository')
         newaction(_("&Import Patches..."), self._repofwd('thgimport'),
                   icon='hg-import', enabled='repoopen', menu='repository')
@@ -334,7 +339,8 @@ class Workbench(QMainWindow):
         newaction(_("&Terminal"), self.terminal, shortcut='Shift+Ctrl+T',
                   icon='utilities-terminal', enabled='repoopen',
                   menu='repository')
-        newaction(_("&Web Server"), self.serve, menu='repository')
+        newaction(_("&Web Server"), self.serve, menu='repository',
+                  icon='hg-serve')
 
         newaction(_("&Help"), self.onHelp, menu='help', icon='help-browser')
         newaction(_("E&xplorer Help"), self.onHelpExplorer, menu='help')
@@ -345,8 +351,7 @@ class Workbench(QMainWindow):
                   visible=visiblereadme, shortcut='Ctrl+F1')
         newseparator(menu='help')
         newaction(_("About &Qt"), QApplication.aboutQt, menu='help')
-        newaction(_("&About TortoiseHg"), self.onAbout, menu='help',
-                  icon='thg-logo')
+        newaction(_("&About TortoiseHg"), self.onAbout, menu='help', icon='thg')
 
         newaction(_('&Incoming'), data='incoming', icon='hg-incoming',
                   enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+,')
@@ -438,7 +443,7 @@ class Workbench(QMainWindow):
         self.urlCombo.blockSignals(False)
         self._updateSyncUrlToolTip(self.urlCombo.currentIndex())
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _setupUrlComboIfCurrent(self, root):
         w = self._currentRepoWidget()
         if w.repoRootPath() == root:
@@ -646,7 +651,7 @@ class Workbench(QMainWindow):
         self._setConsoleTaskTabVisible(not docked and visible)
         self._updateShowConsoleAction()
 
-    @pyqtSlot(QString, bool)
+    @pyqtSlot(str, bool)
     def openRepo(self, root, reuse, bundle=None):
         """Open tab of the specified repo [unicode]"""
         root = unicode(root)
@@ -657,12 +662,12 @@ class Workbench(QMainWindow):
         if not self.repoTabsWidget.openRepo(root, bundle):
             return
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def showRepo(self, root):
         """Activate the repo tab or open it if not available [unicode]"""
         self.openRepo(root, True)
 
-    @pyqtSlot(unicode, QString)
+    @pyqtSlot(str, str)
     def setRevsetFilter(self, path, filter):
         if self.repoTabsWidget.selectRepo(path):
             w = self.repoTabsWidget.currentWidget()
@@ -729,6 +734,7 @@ class Workbench(QMainWindow):
                 a.setChecked(False)
             if self.actionSelectTaskPbranch is not None:
                 self.actionSelectTaskPbranch.setVisible(False)
+            self.lockToolAction.setVisible(False)
         else:
             exts = repoWidget.repo.extensions()
             if self.actionSelectTaskPbranch is not None:
@@ -736,6 +742,7 @@ class Workbench(QMainWindow):
             name = repoWidget.currentTaskTabName()
             for action in self.actionGroupTaskView.actions():
                 action.setChecked(str(action.data().toString()) == name)
+            self.lockToolAction.setVisible('simplelock' in exts)
         self._updateShowConsoleAction()
 
         for i, a in enumerate(a for a in self.actionGroupTaskView.actions()
@@ -755,7 +762,7 @@ class Workbench(QMainWindow):
         self._updateMenu()
         self._updateWindowTitle()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _onCurrentRepoChanged(self, curpath):
         curpath = unicode(curpath)
         self._console.setCurrentRepoRoot(curpath or None)
@@ -837,7 +844,7 @@ class Workbench(QMainWindow):
     def _updateAbortAction(self, repoagent):
         self.actionAbort.setEnabled(repoagent.isBusy())
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _onBusyChanged(self, root):
         repoagent = self._repomanager.repoAgent(root)
         self._updateAbortAction(repoagent)
@@ -867,7 +874,7 @@ class Workbench(QMainWindow):
         if ok:
             self.gotorev(rev)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def gotorev(self, rev):
         w = self._currentRepoWidget()
         if w:
@@ -887,7 +894,7 @@ class Workbench(QMainWindow):
             self.openRepo(dlg.destination(), False)
 
     @pyqtSlot()
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def cloneRepository(self, uroot=None):
         """ Run clone dialog """
         # it might be better to reuse existing CloneDialog
@@ -904,8 +911,10 @@ class Workbench(QMainWindow):
         dlg.clonedRepository.connect(self._openClonedRepo)
         return dlg
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def _openClonedRepo(self, root, sourceroot):
+        root = unicode(root)
+        sourceroot = unicode(sourceroot)
         self.reporegistry.addClonedRepo(root, sourceroot)
         self.showRepo(root)
 
