@@ -7,7 +7,7 @@
 
 import os, socket
 
-from mercurial import cmdutil, extensions, sslutil, util
+from mercurial import cmdutil, commands, extensions, sslutil, util
 
 from tortoisehg.util import hgversion
 from tortoisehg.util.i18n import agettext as _
@@ -38,10 +38,7 @@ def debuggethostfingerprint(ui, repo, source='default'):
     sock = socket.socket()
     try:
         sock.connect((host, port))
-        sock = sslutil.ssl_wrap_socket(sock, None, None, serverhostname=host)
-        if not util.safehasattr(sock, 'getpeercert'):  # python 2.5 ?
-            raise util.Abort(_('host fingerprint for %s cannot be obtained '
-                               '(Python too old)') % host)
+        sock = sslutil.wrapsocket(sock, None, None, ui, serverhostname=host)
         peercert = sock.getpeercert(True)
         if not peercert:
             raise util.Abort(_('%s certificate error: no certificate received')
@@ -101,11 +98,11 @@ def qreorder(ui, repo, *patches, **opts):
     """move patches to the beginning or after the specified patch"""
     after = opts['after'] or None
     q = repo.mq
-    if util.any(n not in q.series for n in patches):
+    if any(n not in q.series for n in patches):
         raise util.Abort(_('unknown patch to move specified'))
     if after in patches:
         raise util.Abort(_('invalid patch position specified'))
-    if util.any(q.isapplied(n) for n in patches):
+    if any(q.isapplied(n) for n in patches):
         raise util.Abort(_('cannot move applied patches'))
 
     if after is None:
@@ -131,3 +128,8 @@ def uisetup(ui):
         cmdtable.update(_mqcmdtable)
     except KeyError:
         pass
+
+    # ignore --no-commit on hg<3.7 (ce76c4d2b85c)
+    _aliases, entry = cmdutil.findcmd('backout', commands.table)
+    if not any(op for op in entry[1] if op[1] == 'no-commit'):
+        entry[1].append(('', 'no-commit', None, '(EXPERIMENTAL)'))
