@@ -6,7 +6,8 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 try:
-    from config import icon_path, bin_path, license_path, locale_path
+    from tortoisehg.util.config import (icon_path, bin_path, license_path,
+                                        locale_path)
 except ImportError:
     icon_path, bin_path, license_path, locale_path = None, None, None, None
 
@@ -58,8 +59,6 @@ def get_hg_command():
     return _hg_command
 
 if os.name == 'nt':
-    import _winreg
-    import win32net
     import win32file
 
     def find_in_path(pgmname):
@@ -103,13 +102,14 @@ if os.name == 'nt':
 
     def get_prog_root():
         if getattr(sys, 'frozen', False):
-            try:
-                return _winreg.QueryValue(_winreg.HKEY_LOCAL_MACHINE,
-                                          r"Software\TortoiseHg")
-            except:
-                pass
+            return os.path.dirname(sys.executable)
         return os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
 
+    def get_thg_command():
+        if getattr(sys, 'frozen', False):
+            return [sys.executable]
+        return [sys.executable] + sys.argv[:1]
+
     def is_unc_path(path):
         unc, rest = os.path.splitunc(path)
         return bool(unc)
@@ -124,25 +124,6 @@ if os.name == 'nt':
         else:
             return False
 
-    USE_OK  = 0     # network drive status
-
-    def netdrive_status(drive):
-        """
-        return True if a network drive is accessible (connected, ...),
-        or False if <drive> is not a network drive
-        """
-        if hasattr(os.path, 'splitunc'):
-            unc, rest = os.path.splitunc(drive)
-            if unc: # All UNC paths (\\host\mount) are considered nonlocal
-                return True
-        letter = os.path.splitdrive(drive)[0].upper()
-        _drives, total, _ = win32net.NetUseEnum(None, 1, 0)
-        for drv in _drives:
-            if drv['local'] == letter:
-                info = win32net.NetUseGetInfo(None, letter, 1)
-                return info['status'] == USE_OK
-        return False
-
 else: # Not Windows
 
     def find_in_path(pgmname):
@@ -171,12 +152,8 @@ else: # Not Windows
         path = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
         return path
 
-    def netdrive_status(drive):
-        """
-        return True if a network drive is accessible (connected, ...),
-        or False if <drive> is not a network drive
-        """
-        return False
+    def get_thg_command():
+        return sys.argv[:1]
 
     def is_unc_path(path):
         return False
