@@ -84,7 +84,7 @@ def _packmsgs(msgs, label):
     >>> _packmsgs(['foo ', _labeledstr('bar', 'log.branch')], '')
     ['foo ', '\x01log.branch\nbar']
     """
-    if not util.any(isinstance(e, _labeledstr) for e in msgs):
+    if not any(isinstance(e, _labeledstr) for e in msgs):
         # pack into single message to avoid overhead of label header and
         # channel protocol; also it's convenient for command-server client
         # to receive the whole message at once.
@@ -225,10 +225,9 @@ def _extenduiclass(parcls):
             except ValueError:
                 raise util.Abort(_('unrecognized response: %s') % r)
 
-        # getpass() cannot communicate with command server as of hg 2.9
         def getpass(self, prompt=None, default=None):
             prompt = self.label(prompt or _('password: '), 'ui.getpass')
-            return self.prompt(prompt, default)
+            return super(pipeui, self).getpass(prompt, default)
 
         def progress(self, topic, pos, item='', unit='', total=None):
             now = time.time()
